import cv2          
import numpy as np
import matplotlib.pyplot as plt


def mono2rgb(img):                   #    モノカラーからRGBへ
    img3 = np.empty((*img.shape, 3))
    for i in range(len(img)):
        for j in range(len(img[0])):
            for k in range(3):
                img3[i][j][k] = img[i][j]
    return img3

f_img = input('Input figure file (*.png) = ')
img1 = cv2.imread(f_img)                        #   画像の読み込み
img1 = cv2.resize(img1, dsize=(400,400))        #   400×400へのサイズ変換
#
#       画像のモノクロ化
#
img0 = np.empty((400, 400))
for i in range(400):
    for j in range(400):
        v = 0
        for k in range(3):
            v += img1[i][j][k]
        img0[i][j] = v / (255*3)

plt.imshow(mono2rgb(img0))     #   変換画像の表示
plt.title('Transformed Input Figure')
plt.show()
#
#　　　　　　　画像の特異値分解
#
u, s, vh = np.linalg.svd(img0, full_matrices=False)
#
#        特異値のプロット
#
plt.plot(range(len(s)), s)
plt.xticks([0, len(s)-1], ['1', f'{len(s)}'])
plt.ylabel('Singular value')
plt.show()

plt.plot(range(20), s[:20])
plt.xticks([0, 19], ['1', '20'])
plt.ylabel('Singular value')
plt.show()

imgs = []
for h in range(5):
    print('h =', h)
    if h < 4:
        #     h番目の特異値による分解
        f1 = u[:,h:h+1] @ np.diag(s[h:h+1]) @ vh[h:h+1]
    else:
        #     1番目から4番目までの特異値による分解の和
        f1 = u[:,:4] @ np.diag(s[:4]) @ vh[:4]
    img_1 = np.empty((400, 400, 3), dtype=float)
    min_f1 = np.min(f1)
    if min_f1 < 0.0:
        f1 -= min_f1
    max_f1 = np.max(f1)
    if max_f1 > 1.0:
        f1 /= max_f1
    for i in range(len(f1)):
        for j in range(len(f1[0])):
            for k in range(3):
                img_1[i][j][k] = f1[i][j]
    imgs.append(img_1)
#
#         描画用大画面
#
img_g = np.full((800, 1200, 3), 0.5, dtype=float)

def insert_img(startx0, starty0, img_v):   #   大画面への画像の挿入
    for i in range(400):
        for j in range(400):
            for k in range(3):
                img_g[startx0+i][starty0+j][k] = img_v[i][j][k]

insert_img(0, 0, mono2rgb(img0))
for h in range(5):
    if h < 2:      #     １行目に表示
        insert_img(0, 400*(h+1), imgs[h])
    else:          #     ２行目に表示
        insert_img(400, 400*(h-2), imgs[h])

plt.figure(figsize=(12,8))
plt.imshow(img_g)
plt.show()
